import typer
from typing import Optional

from e2c_client import E2CClient

monitor_app = typer.Typer(short_help="E2C monitoring related commands")


@monitor_app.callback()
def monitor_app_callback():
    """
    Perform monitoring requests for E2C-controlled devices.

    Usage:

    \b
    e2c-client monitor --help
    e2c-client monitor request <host> <channel> <node> <RCA>
    """


@monitor_app.command("request", short_help="Perform AMB monitor request")
def request(
    host: Optional[str] = typer.Option(None, help="E2C hostname or IP address"),
    port: Optional[int] = typer.Option(2000, help="E2C socket server port"),
    channel: Optional[str] = typer.Option(None, help="LRU channel number"),
    node: Optional[str] = typer.Option(None, help="LRU node address"),
    rca: Optional[str] = typer.Option(None, help="Monitor request RCA"),
):
    try:
        _channel = int(channel[2:], base=16) if channel.lower().startswith("0x") else int(channel)
        _node = int(node[2:], base=16) if node.lower().startswith("0x") else int(node)
        _rca = int(rca[2:], base=16) if rca.lower().startswith("0x") else int(rca)
        client = E2CClient(host, port)
        error, data = client.send_request(
            resource_id=0, bus_id=_channel, node_id=_node, can_addr=_rca, mode=0, data=b""
        )
        print(f"Error: {error}")
        print("Data: ")
        print(", ".join(hex(b) for b in data))
    except Exception as e:
        print(f"Error: {str(e)}")


@monitor_app.command("temperature", short_help="Perform AMB monitor request for AMBSI temperature")
def temperature(
    host: Optional[str] = typer.Option(None, help="E2C hostname or IP address"),
    port: Optional[int] = typer.Option(2000, help="E2C socket server port"),
    channel: Optional[str] = typer.Option(None, help="LRU channel number"),
    node: Optional[str] = typer.Option(None, help="LRU node address"),
):
    try:
        _channel = int(channel[2:], base=16) if channel.lower().startswith("0x") else int(channel)
        _node = int(node[2:], base=16) if node.lower().startswith("0x") else int(node)
        client = E2CClient(host, port)
        temp = client.get_temperature_c(_channel, _node)
        print(f"Temperature: {temp} [C]")
    except Exception as e:
        print(f"Error: {str(e)}")

@monitor_app.command("nodes", short_help="List all CAN nodes reported by E2C host")
def nodes(
    host: Optional[str] = typer.Option(None, help="E2C hostname or IP address"),
    port: Optional[int] = typer.Option(2000, help="E2C socket server port"),
):
    try:
        client = E2CClient(host, port)
        client.get_all_nodes()
    except Exception as e:
        print(f"Error: {str(e)}")
